#!/bin/sh
#
# Copyright (C) 1998-2000 VMware, Inc.  All Rights Reserved.
#
# This script manages the services needed to run VMware

# Basic support for IRIX style chkconfig
# chkconfig: 3 90 08
# description: Manages the services needed to run VMware

# BEGINNING_OF_TMPDIR_DOT_SH
#!/bin/sh

# Create a temporary directory
#
# They are a lot of small utility programs to create temporary files in a
# secure way, but none of them is standard. So I wrote this --hpreg
make_tmp_dir() {
  local dirname="$1" # OUT
  local prefix="$2"  # IN
  local tmp
  local serial
  local loop

  tmp="${TMPDIR:-/tmp}"

  # Don't overwrite existing user data
  # -> Create a directory with a name that didn't exist before
  #
  # This may never succeed (if we are racing with a malicious process), but at
  # least it is secure
  serial=0
  loop='yes'
  while [ "$loop" = 'yes' ]; do
    # Check the validity of the temporary directory. We do this in the loop
    # because it can change over time
    if [ ! -d "$tmp" ]; then
      echo 'Error: "'"$tmp"'" is not a directory.'
      echo
      exit 1
    fi
    if [ ! -w "$tmp" -o ! -x "$tmp" ]; then
      echo 'Error: "'"$tmp"'" should be writable and executable.'
      echo
      exit 1
    fi

    # Be secure
    # -> Don't give write access to other users (so that they can not use this
    # directory to launch a symlink attack)
    if mkdir -m 0755 "$tmp"'/'"$prefix$serial" >/dev/null 2>&1; then
      loop='no'
    else
      serial=`expr "$serial" + 1`
      if [ `expr "$serial" % 200` = '0' ]; then
        echo 'Warning: The "'"$tmp"'" directory may be under attack.'
        echo
      fi
    fi
  done

  eval "$dirname"'="$tmp"'"'"'/'"'"'"$prefix$serial"'
}

# END_OF_TMPDIR_DOT_SH

vmware_etc_dir=/etc/vmware

#
# Make the old installer aware of the new installer.sh mechanism.
# Kludge time :)
#

if [ "$1" = "stop" ]; then
    pid="$$"
    ppid=`grep "^PPid:" /proc/"$pid"/status | cut -f 2`
    parent_name=`grep "^Name:" /proc/"$ppid"/status | cut -f 2`
    if [ "$parent_name" = "install.pl" ]; then
	# The old installer called this script
	vmware_installer="$vmware_etc_dir"/installer.sh
        if [ -x "$vmware_installer" ]; then
	    echo 'A previous installation of VMware has been detected.'
	    echo

	    kind=`"$vmware_installer" kind`
	    echo 'The previous installation was made by the '"$kind"' installer,'
	    echo 'which is more recent than this installer.'
	    echo 'This means that your are trying to downgrade or remove VMware.'
	    echo -n 'Do you really want to do this? (yes/no) [n] '
	    read answer
            echo
	    case "$answer" in
		y|Y|yes|Yes|YES)
                    make_tmp_dir 'tmpdir' 'vmware-installer'
		    "$vmware_installer" convertdb old "$tmpdir"'/prev_db.tar.gz'
		    "$vmware_installer" uninstall
		    if [ ! "$?" -eq 0 ]; then
		       rm -rf "$tmpdir"
                       echo
                       echo 'Failure'
                       echo
		       exit 1
                    fi
                    made_dir='0'
                    if [ ! -d '/etc/vmware' ]; then
                      mkdir -m 0755 '/etc/vmware'
                      made_dir='1'
                    fi
                    tar -C / -xzopf "$tmpdir"'/prev_db.tar.gz'
                    rm -rf "$tmpdir"
                    if [ "$made_dir" = '1' ]; then
                      echo 'directory /etc/vmware' >> '/etc/vmware/locations'
                    fi
		    
		    exit 0
		    ;;
		    
		*)
		    # Prevent the display of the old installer's error message
		    kill "$ppid"
		    exit 1
		    ;;
	    esac
        else
	    echo 'Error: Unable to find the previous installer object '"$vmware_installer"'.'
	    echo

	    # Exit with 0 otherwise the user will never be able to re-install
            exit 0
	fi
    fi
fi

#
# From now on, this is a normal startup script. I swear.
#

# Since this script is installed, our main database should be installed too and
# should contain the basic information
vmware_db="$vmware_etc_dir"/locations
if [ ! -r "$vmware_db" ]; then
    echo 'Warning: Unable to find VMware'"'"'s main database '"$vmware_db"'.'
    echo	    
    
    exit 1
fi

# BEGINNING_OF_DB_DOT_SH
#!/bin/sh

#
# Manage an installer database
#

# Load an answer from a database on disk
db_load_answer() {
  local dbfile="$1" # IN
  local id="$2"     # IN

  grep '^remove_answer '"$id"'$\|^answer '"$id"' ' "$dbfile" | tail -1 | sed -n 's/^answer '"$id"' //p'
}

# Load all answers from a database on disk to memory (<dbvar>_answer_*
# variables)
db_load() {
  local dbvar="$1"  # OUT
  local dbfile="$2" # IN
  local value
  local id
  local answers
  local answers2

  answers=''
  if [ -r "$dbfile" ]; then
    for id in `sed -n -e 's/^answer \(.*\) .*/\1/p' "$dbfile"`; do
      answers2=`echo "$answers" | sed 's/ '"$id"' //'`
      if [ "$answers2" = "$answers" ]; then
        value=`db_load_answer "$dbfile" "$id"`
        if [ "$value" != '' ]; then
          eval "$dbvar"'_answer_'"$id"'="$value"'
          answers="$answers"' '"$id"' '
        fi
      fi
    done
  fi
  eval "$dbvar"'_answers="$answers"'
}

# Iterate through all answers in the database in memory, calling <func> with
# id/value pairs and the remaining arguments to this function
db_iterate() {
  local dbvar="$1" # IN
  local func="$2"  # IN
  shift 2
  local answers
  local i
  local value

  eval 'answers="$'"$dbvar"'_answers"'
  # There is no double quote around $answers on purpose
  for i in $answers; do
    eval 'value="$'"$dbvar"'_answer_'"$i"'"'
    eval "$func"' "$i" "$value" "$@"'
  done
}

# If it exists in memory, remove an answer from a database (disk and memory)
db_remove_answer() {
  local dbvar="$1"  # IN
  local dbfile="$2" # IN
  local id="$3"     # IN
  local answers
  local answers2

  eval 'answers="$'"$dbvar"'_answers"'
  answers2=`echo "$answers" | sed 's/ '"$id"' //'`
  if [ "$answers2" != "$answers" ]; then
    # The answer is in memory
    eval 'unset '"$dbvar"'_answer_'"$id"
    eval "$dbvar"'_answers="$answers2"'
    echo 'remove_answer '"$id" >> "$dbfile"
  fi
}

# Add an answer to a database (disk and memory)
db_add_answer() {
  local dbvar="$1"  # IN
  local dbfile="$2" # IN
  local id="$3"     # IN
  local value="$4"  # IN

  db_remove_answer "$dbvar" "$dbfile" "$id"
  eval "$dbvar"'_answer_'"$id"'="$value"'
  eval "$dbvar"'_answers="$'"$dbvar"'_answers"'"'"' '"'"'"$id"'"'"' '"'"
  echo 'answer '"$id"' '"$value" >> "$dbfile"
}

# Add a file to a database on disk
# 'file' is the file to put in the database (it may not exist on the disk)
# 'tsfile' is the file to get the timestamp from, '' if no timestamp
db_add_file() {
  local dbfile="$1"
  local file="$2"
  local tsfile="$3"
  local date

  if [ "$tsfile" = '' ]; then
    echo 'file '"$file" >> "$dbfile"
  else
    date=`date -r "$tsfile" '+%s' 2> /dev/null`
    if [ ! "$date" = '' ]; then
      date=' '"$date"
    fi
    echo 'file '"$file$date" >> "$dbfile"
  fi
}

# Add a directory to a database on disk
db_add_dir() {
  local dbfile="$1"
  local dir="$2"

  echo 'directory '"$dir" >> "$dbfile"
}

# END_OF_DB_DOT_SH

vmware_init_dir=`db_load_answer "$vmware_db" 'INITDIR'`
# This defines echo_success() and echo_failure() on RedHat
if [ -r "$vmware_init_dir"/init.d/functions ]; then
    . "$vmware_init_dir"/init.d/functions
fi

# This defines $rc_done and $rc_failed on S.u.S.E.
if [ -f /etc/rc.config ]; then
     # Don't quite feel like including the entire file.
     # There could be conflicts.
     rc_done="\033[71G\033[32mdone\033[m"
     rc_failed="\033[71G\033[31m\033[1mfailed\033[m"
else
     rc_done="\033[71G done"
     rc_failed="\033[71Gfailed"
fi

subsys=vmware
driver=vmmon
ppuser=vmppuser
vnet=vmnet
bridge=vmnet-bridge
dhcpd=vmnet-dhcpd
netifup=vmnet-netifup
ping=vmware-ping
smbd=vmware-smbd
nmbd=vmware-nmbd

#
# Utilities
#

# BEGINNING_OF_IPV4_DOT_SH
#!/bin/sh

#
# IPv4 address functions
#
# Thanks to Owen DeLong <owen@delong.com> for pointing me at bash's arithmetic
# expansion ability, which is a lot faster than using 'expr' --hpreg
#

# Compute the subnet address associated to a couple IP/netmask
ipv4_subnet() {
  local ip="$1"
  local netmask="$2"
  local splitter

  # Split a quad-dotted address into 4 bytes
  splitter="$ip"."$netmask"
  # There is no double quote around $splitter on purpose
  set `IFS='.'; echo $splitter`

  echo $(($1 & $5)).$(($2 & $6)).$(($3 & $7)).$(($4 & $8))
}

# Compute the broadcast address associated to a couple IP/netmask
ipv4_broadcast() {
  local ip="$1"
  local netmask="$2"
  local splitter

  # Split a quad-dotted address into 4 bytes
  splitter="$ip"."$netmask"
  # There is no double quote around $splitter on purpose
  set `IFS='.'; echo $splitter`

  echo $(($1 | (255 - $5))).$(($2 | (255 - $6))).$(($3 | (255 - $7))).$(($4 | (255 - $8)))
}

# END_OF_IPV4_DOT_SH

# Are we running in a VM?
inVM() {
    inVM_checkvm="$vmware_etc_dir"/checkvm
    [ -x "$inVM_checkvm" ] || exit 1
    "$inVM_checkvm" >/dev/null 2>&1
}

# Initialize some variables with values from the database
variables_init() {
    vmware_bin_dir=`db_load_answer "$vmware_db" 'BINDIR'`
    vmware_lib_dir=`db_load_answer "$vmware_db" 'LIBDIR'`
    vmware_networking=`db_load_answer "$vmware_db" 'NETWORKING'`
    vmware_bridgeif=`db_load_answer "$vmware_db" 'VNET_INTERFACE'`
    vmware_samba=`db_load_answer "$vmware_db" 'VNET_SAMBA'`
    vmnetHostOnlyAddress=`db_load_answer "$vmware_db" 'VNET_HOSTONLY_HOSTADDR'`
    vmnetHostOnlyNetmask=`db_load_answer "$vmware_db" 'VNET_HOSTONLY_NETMASK'`
}

#
# Count the number of running virtual machines
# by looking at the number of references to the
# $driver module.
#
countVMs() {
    # Beware of module dependancies here. An exact match is important
    /sbin/lsmod | awk 'BEGIN {n = 0;} {if ($1 == "'"$driver"'") n = $3;} END {print n;}'
}

# Is a given module loaded?
isLoaded() {
  local module="$1"

  /sbin/lsmod | awk 'BEGIN {n = "no";} {if ($1 == "'"$module"'") n = "yes";} END {print n;}'
}

# Check if there is an IP route for a given subnet via a given interface
# Return true if there is _NO_ such route
noRoutePresent() {
  local subnet="$1" # IN
  local intf="$2"   # IN
  local matches

  # Beware, there may be several identical routes
  matches=`/sbin/route -n | grep '^'"$subnet"'.*'"$intf"'$'`
  [ "$matches" = '' ]
}

#
# Check that the IP address we are going to assign to the host machine on
# a private IP network does not already exist
#
# NB: If you don't want to do this test, just substitute
#     false for it.
#
lookForHostOnlyNetwork() {
  local ip="$1"

  "$vmware_bin_dir"/"$ping" -q "$ip"
}

vmware_failed() {
   if `type -type echo_failure >/dev/null`; then
      echo_failure
   else
      echo -ne "$rc_failed"
   fi
}

vmware_success() {
   if `type -type echo_success >/dev/null`; then
      echo_success
   else
      echo -ne "$rc_done"
   fi
}

# Execute a macro
vmware_exec() {
  local msg="$1"  # IN
  local func="$2" # IN
  shift 2

  echo -n '   '"$msg"

  # On Caldera 2.2, SIGHUP is sent to all our children when this script exits
  # I wanted to use shopt -u huponexit instead but their bash version
  # 1.14.7(1) is too old
  if [ "$VMWARE_DEBUG" = 'yes' ]; then
    (trap '' SIGHUP; "$func" "$@")
  else
    (trap '' SIGHUP; "$func" "$@") >/dev/null 2>&1
  fi
  if [ "$?" -gt 0 ]; then
    vmware_failed
    echo
    return 1
  fi

  vmware_success
  echo
  return 0
}

# Execute a macro in the background
vmware_bg_exec() {
  local msg="$1"  # IN
  local func="$2" # IN
  shift 2

  if [ "$VMWARE_DEBUG" = 'yes' ]; then
    # Force synchronism when debugging
    vmware_exec "$msg" "$func" "$@"
  else
    echo -n '   '"$msg"' (background)'

    # On Caldera 2.2, SIGHUP is sent to all our children when this script exits
    # I wanted to use shopt -u huponexit instead but their bash version
    # 1.14.7(1) is too old
    (trap '' SIGHUP; "$func" "$@") 2>&1 | logger -t 'VMware[init]' -p daemon.err &

    vmware_success
    echo
    return 0
  fi
}

#
# Macro definitions
#
# Note:
#  . Each daemon must be started from its own directory to avoid busy devices
#  . Each PID file doesn't need to be added to the installer database, because
#    it is going to be automatically removed when it becomes stale (after a
#    reboot). It must go directly under /var/run, or some distributions
#    (RedHat 6.0) won't clean it
#

# Terminate a process synchronously
vmware_synchrone_kill() {
  local pid="$1"    # IN
  local signal="$2" # IN
  local second

  kill -"$signal" "$pid" >/dev/null 2>&1

  # Wait a bit to see if the dirty job has really been done
  for second in 0 1 2 3 4 5 6 7 8 9 10; do
    if [ ! -d /proc/"$pid" ]; then
      # Success
      return 0
    fi

    sleep 1
  done

  # Timeout
  return 1
}

# Kill the process associated to a pidfile
vmware_stop_pidfile() {
  local pidfile="$1" # IN
  local pid

  pid=`cat "$pidfile" 2>/dev/null`
  if [ "$pid" = '' ]; then
    # The file probably does not exist or is empty. Success
    return 0
  fi

  # First try a nice SIGTERM
  if vmware_synchrone_kill "$pid" 15; then
    return 0
  fi

  # Then send a strong SIGKILL
  if vmware_synchrone_kill "$pid" 9; then
    return 0
  fi

  return 1
}

# Start the virtual machine monitor kernel service
vmware_start_vmmon() {
   /sbin/insmod -s -f "$driver" || exit 1

   exit 0
}

# Stop the virtual machine monitor kernel service
vmware_stop_vmmon() {
  if [ `isLoaded "$driver"` = 'yes' ]; then
    /sbin/rmmod "$driver" || exit 1
  fi

  exit 0
}

# Start the virtual machine parallel port kernel service
vmware_start_vmppuser() {
   if ! grep -q ' parport_release[^'$'\t'']*$' /proc/ksyms; then
     # parport support is not built in the kernel
     /sbin/modprobe parport || exit 1
   fi
   if ! grep -q ' parport_pc_[^'$'\t'']*$' /proc/ksyms; then
     # parport_pc support is not built in the kernel
     /sbin/modprobe parport_pc || exit 1
   fi
   /sbin/insmod -s -f "$ppuser" || exit 1

   exit 0
}

# Stop the virtual machine parallel port kernel service
vmware_stop_vmppuser() {
  if [ `isLoaded "$ppuser"` = 'yes' ]; then
    /sbin/rmmod "$ppuser" || exit 1
  fi
  # Try to unload the modules. Failure is allowed because some other process
  # could be using them.
  /sbin/modprobe -r parport_pc >/dev/null 2>&1
  /sbin/modprobe -r parport >/dev/null 2>&1

  # Return the right exitcode even if the previous commands failed
  exit 0
}

# Start the virtual ethernet kernel service
vmware_start_vmnet() {
   /sbin/insmod -s -f "$vnet" || exit 1

   exit 0
}

# Stop the virtual ethernet kernel service
vmware_stop_vmnet() {
  if [ `isLoaded "$vnet"` = 'yes' ]; then
    /sbin/rmmod "$vnet" || exit 1
  fi

  exit 0
}

# Create a virtual host ethernet interface and connect it to a virtual
# ethernet hub
vmware_start_netifup() {
  local vHostIf="$1" # IN
  local vHubNr="$2"  # IN

  cd "$vmware_bin_dir" && "$vmware_bin_dir"/"$netifup" \
    -d /var/run/"$netifup"-"$vHostIf".pid /dev/vmnet"$vHubNr" "$vHostIf"
}

# Disconnect a virtual host ethernet interface from a virtual ethernet hub
# and destroy the virtual host ethernet interface
vmware_stop_netifup() {
  local vHostIf="$1" # IN
  
  vmware_stop_pidfile /var/run/"$netifup"-"$vHostIf".pid
}

# Connect a physical host ethernet interface to a virtual ethernet hub
vmware_start_bridge() {
  local vHubNr="$1"  # IN
  local pHostIf="$2" # IN

  cd "$vmware_bin_dir" && "$vmware_bin_dir"/"$bridge" \
    -d /var/run/"$bridge"-"$vHubNr".pid /dev/vmnet"$vHubNr" "$pHostIf"
}

# Disconnect a physical host ethernet interface from a virtual ethernet hub
vmware_stop_bridge() {
  local vHubNr="$1"  # IN

  vmware_stop_pidfile /var/run/"$bridge"-"$vHubNr".pid
}

# Start a SMB name server on a private IP network
vmware_start_nmbd() {
  local vHostIf="$1" # IN

  # Disable logging to avoid the uncontrolled creation of unmanaged files
  cd "$vmware_bin_dir" && "$vmware_bin_dir"/"$nmbd" -D -l /dev/null \
    -s "$vmware_etc_dir"/"$vHostIf"/smb/smb.conf \
    -f /var/run/"$nmbd"-"$vHostIf".pid
}

# Stop a SMB name server on a private IP network
vmware_stop_nmbd() {
  local vHostIf="$1" # IN
  
  vmware_stop_pidfile /var/run/"$nmbd"-"$vHostIf".pid
}

# Start a SMB share server on a private IP network
vmware_start_smbd() {
  local vHostIf="$1" # IN

  # Disable logging to avoid the uncontrolled creation of unmanaged files
  cd "$vmware_bin_dir" && "$vmware_bin_dir"/"$smbd" -D -l /dev/null \
    -s "$vmware_etc_dir"/"$vHostIf"/smb/smb.conf \
    -f /var/run/"$smbd"-"$vHostIf".pid
}

# Stop a SMB share server on a private IP network
vmware_stop_smbd() {
  local vHostIf="$1" # IN
  
  vmware_stop_pidfile /var/run/"$smbd"-"$vHostIf".pid
}

# Start a DHCP server on a private IP network
vmware_start_dhcpd() {
  local vHostIf="$1" # IN
  
  # The daemon already logs its output in the system log, so we can safely
  # trash it
  cd "$vmware_bin_dir" && "$vmware_bin_dir"/"$dhcpd" \
    -cf "$vmware_etc_dir"/"$vHostIf"/dhcpd/dhcpd.conf \
    -lf "$vmware_etc_dir"/"$vHostIf"/dhcpd/dhcpd.leases \
    -pf /var/run/"$dhcpd"-"$vHostIf".pid "$vHostIf" >/dev/null 2>&1
}

# Stop a DHCP server on a private IP network
vmware_stop_dhcpd() {
  local vHostIf="$1" # IN
  
  vmware_stop_pidfile /var/run/"$dhcpd"-"$vHostIf".pid
}

# Start the host-only network user service
vmware_start_hostonly() {
  local vHubNr="$1"    # IN
  local vHostIf="$2"   # IN
  local ifIp="$3"      # IN
  local ifMask="$4"    # IN
  local run_dhcpd="$5" # IN
  local run_samba="$6" # IN
  local ifNet

  #
  # Do a cursory check to see if the host-only network
  # configuration is still ok.  We do this so that mobile
  # hosts don't get setup at install time and then moved to
  # a new locale where the host-only network config is no
  # longer valid.
  #
  # NB: This really needs to be done at power-on time when
  #     VM is configured to use host-only networking so that
  #     we aren't fooled by dynamic changes in the network.
  #
  # XXX ping takes 10 seconds to timeout if noone answers;
  #	  that slows boot too much so we do this bit in the
  #	  background.
  #
  if lookForHostOnlyNetwork "$ifIp"; then
    echo 'Host-only networking disabled because '"$ifIp"
    echo 'appears to be a real, physical, existing address.'
    echo 'Please run "'"$vmware_bin_dir"'/vmware-config.pl" to'
    echo 'modify your host-only network configuration.'
    exit 1
  fi

  vmware_start_netifup "$vHostIf" "$vHubNr" || exit 1

  # Configure the virtual host ethernet interface and define the private IP
  # network
  #
  # . We provide the broadcast address explicitly because versions of ifconfig
  #   prior to 1.39 (1999-03-18) seem to miscompute it
  # . 2.0.x kernels don't install a route when the interface is marked up, but
  #   2.2.x kernel do. Since we want to see any errors from route we don't
  #   just discard messages from route, but instead check if the route got
  #   installed before manually adding one.
  ifNet=`ipv4_subnet "$ifIp" "$ifMask"`
  if    ifconfig "$vHostIf" inet "$ifIp" netmask "$ifMask" \
          broadcast `ipv4_broadcast "$ifIp" "$ifMask"` up \
     && noRoutePresent "$ifNet" "$vHostIf"; then
     route add -net "$ifNet" netmask "$ifMask" "$vHostIf"
  fi

  if [ "$run_dhcpd" = 'yes' ]; then
    vmware_start_dhcpd "$vHostIf" || exit 1
  fi

  if [ "$run_samba" = 'yes' ]; then
    vmware_start_nmbd "$vHostIf" || exit 1
    vmware_start_smbd "$vHostIf" || exit 1
  fi

  exit 0
}

# Stop the host-only network user service
vmware_stop_hostonly() {
  local vHostIf="$1"   # IN
  local ifIp="$2"      # IN
  local ifMask="$3"    # IN
  local ifNet
  local ifPresent

  # Terminate the private network
  ifNet=`ipv4_subnet "$ifIp" "$ifMask"`
     noRoutePresent "$ifNet" "$vHostIf" \
  || route del -net "$ifNet" netmask "$ifMask" || exit 1
  # To test if the interface exists, we can not just look at the exitcode
  # because old versions of ifconfig don't exit with 1 when invoked with a
  # non-existing interface
  ifPresent=`ifconfig "$vHostIf" 2>/dev/null`
  if [ "$ifPresent" != '' ]; then
    ifconfig "$vHostIf" down || exit 1
  fi

  vmware_stop_netifup "$vHostIf" || exit 1

  exit 0
}

# See how we were called.
case "$1" in
  start)
        if [ -e "$vmware_etc_dir"/not_configured ]; then
	   echo 'VMware is installed, but it has not been (correctly) configured'
           echo 'for the running kernel. To (re-)configure it, invoke the'
           echo 'following command: '"$vmware_bin_dir"'/vmware-config.pl.'
           echo

           exit 1
        fi

        if `inVM`; then
            # Refuse to start services in a VM: they are useless
            exit 1
        fi

	# VMware is configured. Fetch useful information from the database
        variables_init

	echo "Starting VMware services:"
        exitcode='0'

	vmware_exec 'Virtual machine monitor' vmware_start_vmmon
        exitcode=`expr "$exitcode" + "$?"`

        # The module is not installed on kernels 2.0.x
        if [    -r /lib/modules/preferred/misc/"$ppuser".o \
             -o -r /lib/modules/`uname -r`/misc/"$ppuser".o ]; then
	   vmware_exec 'Virtual bidirectional parallel port' \
             vmware_start_vmppuser
           exitcode=`expr "$exitcode" + "$?"`
        fi

	if [ "$vmware_networking" = "yes" ]; then
	    vmware_exec 'Virtual ethernet' vmware_start_vmnet
	    exitcode=`expr "$exitcode" + "$?"`

	    vmware_exec 'Bridged networking on /dev/vmnet0' \
              vmware_start_bridge '0' "$vmware_bridgeif"
	    exitcode=`expr "$exitcode" + "$?"`

	    if [    -n "$vmnetHostOnlyAddress" \
                 -a -n "$vmnetHostOnlyNetmask" ]; then
                vmware_bg_exec 'Host-only networking on /dev/vmnet1' \
                  vmware_start_hostonly '1' 'vmnet1' "$vmnetHostOnlyAddress" \
                  "$vmnetHostOnlyNetmask" 'yes' "$vmware_samba"
                exitcode=`expr "$exitcode" + "$?"`
	    fi
	fi

        if [ "$exitcode" -gt 0 ]; then
          # Set the 'not configured' flag
          touch "$vmware_etc_dir"'/not_configured'
          chmod 644 "$vmware_etc_dir"'/not_configured'
          db_add_file "$vmware_db" "$vmware_etc_dir"'/not_configured' \
            "$vmware_etc_dir"'/not_configured'
          exit 1
        fi

	[ -d /var/lock/subsys ] || mkdir -p /var/lock/subsys
	touch /var/lock/subsys/"$subsys"
	;;

  stop)
        variables_init

	# look for running vm's
	nvms=`countVMs`
	if [ "$nvms" -gt 0 ]; then
	    if [ "$nvms" -gt 1 ]; then
	        echo "There are $nvms VMs still running.  Please shut them down first."
	    else
	        echo "There is one VM still running.  Please shut it down first."
	    fi
	    exit 1;
	fi

	echo "Stopping VMware services:"
        exitcode='0'

	vmware_exec 'Virtual machine monitor' vmware_stop_vmmon
        exitcode=`expr "$exitcode" + "$?"`

        # The module is not installed on kernels 2.0.x
        if [    -r /lib/modules/preferred/misc/"$ppuser".o \
             -o -r /lib/modules/`uname -r`/misc/"$ppuser".o ]; then
           vmware_exec 'Virtual bidirectional parallel port' \
             vmware_stop_vmppuser
           exitcode=`expr "$exitcode" + "$?"`
        fi

	if [ "$vmware_networking" = "yes" ]; then
	    # NB: must kill off processes using vmnet before
	    #     unloading module

            vmware_exec 'Bridged networking on /dev/vmnet0' vmware_stop_bridge '0'
            exitcode=`expr "$exitcode" + "$?"`

            if [    -n "$vmnetHostOnlyAddress" \
                 -a -n "$vmnetHostOnlyNetmask" ]; then
               vmware_exec 'DHCP server on /dev/vmnet1' vmware_stop_dhcpd \
                 'vmnet1'
               exitcode=`expr "$exitcode" + "$?"`

               if [ "$vmware_samba" = "yes" ]; then
                  vmware_exec 'SMB share server on /dev/vmnet1' \
                    vmware_stop_smbd 'vmnet1'
                  exitcode=`expr "$exitcode" + "$?"`

                  vmware_exec 'SMB name server on /dev/vmnet1' \
                    vmware_stop_nmbd 'vmnet1'
                  exitcode=`expr "$exitcode" + "$?"`
               fi

               vmware_exec 'Host-only networking on /dev/vmnet1' \
                 vmware_stop_hostonly 'vmnet1' "$vmnetHostOnlyAddress" \
                 "$vmnetHostOnlyNetmask"
               exitcode=`expr "$exitcode" + "$?"`
            fi

	    vmware_exec 'Virtual ethernet' vmware_stop_vmnet
            exitcode=`expr "$exitcode" + "$?"`
        fi

        if [ "$exitcode" -gt 0 ]; then
           exit 1
        fi

	rm -f /var/lock/subsys/"$subsys"
	;;

  status)
	# look for running vm's
	nvms=`countVMs`
	if [ "$nvms" -gt 0 ]; then
	    echo "There are $nvms virtual machines running"
	else
	    echo "There are no virtual machines running"
	fi

	if [ "$vmware_networking" = "yes" ]; then
            status "$bridge"
            status "$dhcpd"
            status "$netifup"
        fi

	echo -n "Module $driver "
	/sbin/modprobe "$driver" >/dev/null 2>&1 && echo installed || echo "not installed"
	echo -n "Module $ppuser "
	/sbin/modprobe "$ppuser" >/dev/null 2>&1 && echo installed || echo "not installed"
	if [ "$vmware_networking" = "yes" ]; then
            echo -n "Module $vnet "
            /sbin/modprobe "$vnet" >/dev/null 2>&1 && echo installed || echo "not installed"
        fi
	;;

  restart)
	"$0" stop && "$0" start
	;;

  *)
	echo "Usage: `basename "$0"` {start|stop|status|restart}"
	exit 1
esac

exit 0
